use utf8;
use strict;
use Class::MOP;

binmode(STDOUT, ":utf8");

# リスト1
{
    package Person;

    # クラスの作成
    my $meta = Class::MOP::Class->create('Person');

    # アトリビュートの追加
    $meta->add_attribute(
        'name',
        accessor => 'name',
    );

    # メソッドの追加
    $meta->add_method( describe => sub {
        my $self = shift;
        my $name = $self->name;
        print "$name";
    } );

    # オブジェクトインスタンスの生成
    my $object = $meta->new_object( name => "Shawn" );
    $object->describe();
    print "\n";
}

# 記事には書かれていないCompanyというクラス
{
    package Company;
    my $meta = Class::MOP::Class->create('Company');

    $meta->add_attribute(
        'name',
        accessor => 'name',
    );
}

# リスト2
{
    package Employee;

    # クラスの作成
    my $meta = Class::MOP::Class->create('Employee');

    # 親クラスの指定
    $meta->superclasses('Person');

    # アトリビュートの追加
    $meta->add_attribute(
        'employer',
        accessor => 'employer',
    );

    # リスト3: メソッドのオーバーライド
    $meta->add_method(
        'describe',
        sub {
            my $self = shift;
            my $company = $self->employer->name;
            print "所属 $company, 社員名 ";

            # あとは親クラスにおまかせ
            $self->next::method();
        }
    );

    my $company = Company->meta->new_object(name => "Acme");

    my $employee = $meta->new_object( name => "Shawn", employer => $company );
    $employee->describe();
    print "\n";
}

# リスト3
{
    package Employee2;

    my $meta = Class::MOP::Class->create('Employee2');
    $meta->superclasses('Person');
    $meta->add_attribute(
        'employer',
        accessor => 'employer',
    );

    $meta->add_before_method_modifier(
        'describe',
        sub {
            my $self = shift;
            my $company = $self->employer->name;
            print "所属 $company 社員名 ";
        },
    );
    my $company = Company->meta->new_object(name => "Acme");

    my $employee = $meta->new_object( name => "Shawn", employer => $company );
    $employee->describe();
    print "\n";
}

# リスト4
{
    # メタクラスの取得
    my $class = Employee->meta;

    # "Employee"
    print $class->name, "\n";
    # "Person"
    print $class->superclasses, "\n";

    # メタアトリビュート
    my $attribute =
        $class->find_attribute_by_name('employer');
    my $accessor  = $attribute->accessor;

    # メタメソッド
    my $method = $class->find_method_by_name( $accessor );

    # "employer"
    print $method->name, "\n";
    # "Employee::employer"
    print $method->fully_qualified_name, "\n";
}

