use utf8;
use strict;

binmode(STDOUT, ":utf8");

# リスト8
BEGIN {
    # リスト9 で使われる前に定義されていないといけないのでBEGIN {}
    # で囲ってありますが、普通は .pmファイルに保存して、useすれば
    # 大丈夫です

    package InstanceTracker;
    use Moose;

    # Moose::Meta::Classの子クラスなので、これもメタクラスである
    extends 'Moose::Meta::Class';

    # 生成したインスタンスの配列
    has instances => (
        is      => 'ro',
        isa     => 'ArrayRef',
        default => sub { [] },
    );

    # インスタンスを生成する 継承したメソッドをオーバーライド
    sub _construct_instance {
        my $self = shift;

        # Moose::Meta::Classの_construct_instance本体がインスタンスを返す
        my $instance = $self->next::method(@_);

        # このインスタンスを見失わないようにする
        push @{ $self->instances }, $instance;

        # newにインスタンスを返る
        return $instance;
    };
}

# リスト9
{
    package User;

    # カストマイズしたメタクラスを利用宣言する
    use Moose -metaclass => 'InstanceTracker';

    has name => (
        is       => 'rw',
        isa      => 'Str',
        required => 1,
    );

    # 生成時に、InstanceTrackerの_construct_instanceに登録したフックを実行する
    my $me     = User->new(name => "SARTAK");
    my $friend = User->new(name => "DMAKI");

    # インスタンス全部を小文字の名前に変換する
    for my $instance (@{ User->meta->instances }) {
        $instance->name( lc($instance->name) );
    }

    print $me->name, "\n";     # sartak
    print $friend->name, "\n"; # dmaki
}
