use utf8;
use strict;

binmode(STDOUT, ":utf8");

# リスト13
{
    package Computer;
    use Moose;
    use MooseX::Method::Signatures;

    # シグネチャなしでも良い
    method try_crash {
        die if rand(100) < 1;
    }

    # $delayは位置個定の引数
    method shutdown (Int $delay) {
        # $selfも自動的に割り当てる
        $self->try_crash;
        sleep $delay;
        exit;
    }

    # 名前付き引数
    method speak (Str :$text, Int :$wpm = 160) {
        system("say", "--rate=$wpm", $text);
    }

    my $laptop = Computer->new;
    $laptop->try_crash;
    $laptop->speak(text => "This is goodbye.", wpm => 500);
    # wpmは160のデフォルト
    $laptop->speak(text => "Sayonara.");

    # shutdownの$delay変数は5にする
    # XXX これを実行してしまうと exit() するのでここではスキップしています
    # $laptop->shutdown(5);
}

# リスト14
{
    my $meta = Computer->meta;

    # MooseX::Method::Signatures::Meta::Methodのインスタンス
    my $shutdown = $meta->get_method('shutdown');
    print $shutdown->signature . "\n"; # "(Int $delay)"

    # シグネチャを表現するオブジェクト
    my $signature = $shutdown->parsed_signature;

    # "1" -- positionalである引数がある
    print $signature->has_positional_params ? "1" : "0", "\n";
    # "0" -- 名前付き引数がない
    print $signature->has_named_params ? "1" : "0", "\n";

    # 各引数を表現するオブジェクトの配列
    my @params = @{ $signature->positional_params };
    my $delay = $params[0];

    print $delay->variable_name, "\n"; # $delay
    print $delay->type_constraints->to_string, "\n"; # "Int"
}


