use utf8;
use strict;

binmode(STDOUT, ":utf8");

# リスト15
{
    package MyWAF::Attribute::Field;
    use Moose;
    use Moose::Util::TypeConstraints;
    extends 'Moose::Meta::Attribute';

    has field_type => (
        is      => 'ro',
        isa     => enum([ 'text', 'password' ]),
        default => 'text',
    );

    has maxlength => (
        is        => 'ro',
        isa       => 'Int',
        predicate => 'has_maxlength',
    );

    sub as_form_field {
        my $self = shift;
        my $html = qq[<input];

        $html .= qq[ name="] . $self->name . qq["];
        $html .= qq[ required] if $self->is_required;
        $html .= qq[ type="] . $self->field_type . qq["];

        if ( $self->has_maxlength ) {
            $html .= qq[ maxlength="] .
                $self->maxlength . qq["];
        }
        $html .= qq{>\n};
        return $html;
    }
}

# リスト16
{
    package User;
    use Moose;

    has name => (
        metaclass => 'MyWAF::Attribute::Field',
        is        => 'ro',
        isa       => 'Str',
        required  => 1,
        maxlength => 10,
    );

    has password => (
        metaclass  => 'MyWAF::Attribute::Field',
        is         => 'ro',
        isa        => 'Str',
        required   => 1,
        field_type => 'password',
    );

    sub generate_form {
        my $self = shift;

        my $html = '';
        for my $attribute ($self->meta->get_all_attributes) {
            $html .= $attribute->as_form_field();
        }
        print $html;
    }

    User->generate_form();
}

